"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declareStore = void 0;
/* eslint-disable @typescript-eslint/ban-types */
const store_1 = require("./store");
/**
 * declare the base interface for create store
 * @example
```ts
type State = {
  id: string;
  name: string;
  isAdmin: boolean
};
const initialState: State = {
  id: '',
  name: '',
  isAdmin: false
};
const createUserStore = declareStore({
  initialState,
  updates: {
    setId: (id: string) => (state) => {
      return {
        ...state,
        id: id,
      };
    },
    setName: (name: string) => (state) => {
      return {
        ...state,
        name: name,
      };
    },
    update: (id: string name: string) => (state) => {
      return {
        ...state,
        id: id,
        name: name,
      };
    },
    setIsAdmin: () => (state) => {
      return {
        ...state,
        isAdmin: true,
      };
    },
  },
});

const userStore1 = createUserStore({ id: '1', name: 'User 1', isAdmin: false });

const userStore2 = createUserStore({ id: '2', name: 'User 2', isAdmin: true });

// OR

const users = [
  createUserStore({id: 1, name: 'User 1'}),
  createUserStore({id: 2, name: 'User 2'}),
]

userStore1.updates.setName('User from store 1');

assets.isEqual(userStore1.get().name, 'User from store 1')

assets.isEqual(userStore2.get().name, 'User 2')

// type of createUserStore
type UserStore = ReturnType<typeof createUserStore>;
```
 */
function declareStore(declaration) {
    const { initialState: baseState, updates, options: baseOptions, } = declaration;
    function factory(initialState, options) {
        const state = initialState === undefined
            ? baseState
            : typeof initialState === 'function'
                ? initialState(baseState)
                : initialState;
        const store = (0, store_1.createStore)(state, {
            ...baseOptions,
            ...options,
        });
        return (0, store_1.withStoreUpdates)(store, updates);
    }
    return Object.assign(factory, { updates });
}
exports.declareStore = declareStore;
//# sourceMappingURL=declareStore.js.map