"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createEffectController = exports.GLOBAL_EFFECT_UNHANDLED_ERROR$ = void 0;
const rxjs_1 = require("rxjs");
const store_1 = require("./store");
const GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT = new rxjs_1.Subject();
exports.GLOBAL_EFFECT_UNHANDLED_ERROR$ = GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.asObservable();
function emitGlobalUnhandledError(effectError) {
    if (GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.observed) {
        GLOBAL_EFFECT_UNHANDLED_ERROR_SUBJECT.next(effectError);
    }
    else {
        console.error('Uncaught error in Effect', effectError);
    }
}
const increaseCount = (count) => count + 1;
const decreaseCount = (count) => (count > 0 ? count - 1 : 0);
function createEffectController() {
    const subscriptions = new rxjs_1.Subscription();
    const event$ = new rxjs_1.Subject();
    const done$ = new rxjs_1.Subject();
    const error$ = new rxjs_1.Subject();
    const pendingCount = (0, store_1.createStore)(0, {
        internal: true,
    });
    subscriptions.add(() => {
        event$.complete();
        done$.complete();
        error$.complete();
        pendingCount.destroy();
    });
    const notifications$ = (0, rxjs_1.merge)(done$.pipe((0, rxjs_1.map)((entry) => ({ type: 'result', ...entry }))), error$.pipe((0, rxjs_1.map)((entry) => ({ type: 'error', ...entry }))));
    return {
        state: {
            done$: done$.asObservable(),
            result$: done$.pipe((0, rxjs_1.map)(({ result }) => result)),
            error$: error$.asObservable(),
            final$: notifications$,
            pending: pendingCount.query((count) => count > 0),
            pendingCount: pendingCount.query(rxjs_1.identity),
        },
        start: () => pendingCount.update(increaseCount),
        next: (result) => done$.next(result),
        complete: () => pendingCount.update(decreaseCount),
        error: (effectError) => {
            if (effectError.origin === 'handler') {
                pendingCount.update(decreaseCount);
            }
            if (error$.observed) {
                error$.next(effectError);
            }
            else {
                emitGlobalUnhandledError(effectError);
            }
        },
        destroy: () => {
            subscriptions.unsubscribe();
        },
    };
}
exports.createEffectController = createEffectController;
//# sourceMappingURL=effectController.js.map