"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declareViewController = exports.declareController = exports.createController = void 0;
const ditox_1 = require("ditox");
const scope_1 = require("./scope");
function createController(factory) {
    const scope = (0, scope_1.createScope)();
    const controller = factory(scope);
    return {
        ...controller,
        destroy: () => {
            var _a;
            (_a = controller.destroy) === null || _a === void 0 ? void 0 : _a.call(controller);
            scope.destroy();
        },
    };
}
exports.createController = createController;
function declareController(tokens, factory) {
    return (0, ditox_1.injectable)((deps) => createController((scope) => factory(deps, scope)), tokens);
}
exports.declareController = declareController;
function declareViewController(tokensOrFactory, factory) {
    return (container, ...params) => {
        if (typeof tokensOrFactory === 'function') {
            return createController((scope) => {
                return tokensOrFactory(scope, ...params);
            });
        }
        return (0, ditox_1.injectable)((dependencies) => {
            return createController((scope) => {
                const factoryValue = factory;
                const result = factoryValue(dependencies, scope);
                if (typeof result === 'function') {
                    return result(scope, ...params);
                }
                return result;
            });
        }, tokensOrFactory)(container);
    };
}
exports.declareViewController = declareViewController;
//# sourceMappingURL=mvc.js.map