"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createScope = void 0;
const rxjs_1 = require("rxjs");
const effect_1 = require("./effect");
const store_1 = require("./store");
/**
 * Creates `Scope` instance.
 */
function createScope() {
    const subscriptions = new rxjs_1.Subscription();
    function registerTeardown(teardown) {
        subscriptions.add(teardown);
    }
    function destroy() {
        subscriptions.unsubscribe();
    }
    function createController(factory) {
        const controller = factory();
        registerTeardown(controller.destroy);
        return controller;
    }
    return {
        add: registerTeardown,
        destroy,
        createController,
        createStore(initialState, options) {
            return createController(() => (0, store_1.createStore)(initialState, options));
        },
        createEffect(handler, options) {
            return createController(() => (0, effect_1.createEffect)(handler, options));
        },
        handle(source, handler, options) {
            const effect = createController(() => (0, effect_1.createEffect)(handler, options));
            effect.handle(source);
            return effect;
        },
        subscribe(source, nextOrObserver) {
            const subscription = source.subscribe(nextOrObserver);
            registerTeardown(subscription);
            return subscription;
        },
    };
}
exports.createScope = createScope;
//# sourceMappingURL=scope.js.map