"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.declareStoreWithUpdates = exports.pipeStore = void 0;
const store_1 = require("./store");
/**
 * Creates a deferred or transformed view of the store.
 */
function pipeStore(store, operator) {
    let subscription;
    const clone = (0, store_1.createStore)(store.get(), {
        internal: true,
        onDestroy: () => {
            if (subscription) {
                subscription.unsubscribe();
                subscription = undefined;
            }
        },
    });
    subscription = store.value$.pipe(operator).subscribe({
        next: (state) => {
            clone.set(state);
        },
        complete: () => {
            clone.destroy();
        },
    });
    return clone;
}
exports.pipeStore = pipeStore;
/**
 * @deprecated Use `declareStore()`
 */
function declareStoreWithUpdates(initialState, updates, baseOptions) {
    return (state = initialState, options) => {
        return (0, store_1.withStoreUpdates)((0, store_1.createStore)(state, { ...baseOptions, ...options }), updates);
    };
}
exports.declareStoreWithUpdates = declareStoreWithUpdates;
//# sourceMappingURL=storeUtils.js.map