import { injectable } from 'ditox';
import { createScope } from './scope';
export function createController(factory) {
    const scope = createScope();
    const controller = factory(scope);
    return {
        ...controller,
        destroy: () => {
            var _a;
            (_a = controller.destroy) === null || _a === void 0 ? void 0 : _a.call(controller);
            scope.destroy();
        },
    };
}
export function declareController(tokens, factory) {
    return injectable((deps) => createController((scope) => factory(deps, scope)), tokens);
}
export function declareViewController(tokensOrFactory, factory) {
    return (container, ...params) => {
        if (typeof tokensOrFactory === 'function') {
            return createController((scope) => {
                return tokensOrFactory(scope, ...params);
            });
        }
        return injectable((dependencies) => {
            return createController((scope) => {
                const factoryValue = factory;
                const result = factoryValue(dependencies, scope);
                if (typeof result === 'function') {
                    return result(scope, ...params);
                }
                return result;
            });
        }, tokensOrFactory)(container);
    };
}
//# sourceMappingURL=mvc.js.map