export function createStoreLoggerExtension(logger) {
    return (api) => ({
        onStoreEvent(event) {
            var _a, _b;
            const { type, store } = event;
            const storeName = `${(_a = store.name) !== null && _a !== void 0 ? _a : ''}#${store.id}`;
            switch (type) {
                case 'created':
                case 'destroyed': {
                    logger(storeName, type);
                    break;
                }
                case 'updated': {
                    logger(storeName, type, {
                        nextState: event.nextState,
                        prevState: event.prevState,
                    });
                    break;
                }
                case 'mutation': {
                    const mutationName = (_b = api.getStateMutationMetadata(event.mutation).name) !== null && _b !== void 0 ? _b : 'anonymous';
                    logger(storeName, type, mutationName, {
                        nextState: event.nextState,
                        prevState: event.prevState,
                    });
                }
            }
        },
    });
}
//# sourceMappingURL=storeLoggerExtension.js.map